'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _getScripts = require('../utils/get-scripts');

var _getScripts2 = _interopRequireDefault(_getScripts);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var metadataCache = {};

function toKeyValuePair(object) {
  return Object.keys(object).map(function (key) {
    return { key: key, value: object[key] };
  });
}

function getCacheOrRequire(packagePath) {
  if (metadataCache[packagePath]) {
    return metadataCache[packagePath];
  }

  var metadata = require(packagePath);
  metadataCache[packagePath] = metadata;
  return metadata;
}

function loadMetadata(dep, dir) {
  try {
    var packagePath = _path2.default.resolve(dir, 'node_modules', dep, 'package.json');
    return getCacheOrRequire(packagePath);
  } catch (error) {
    return {}; // ignore silently
  }
}

function toMetadata(dep, dir) {
  var metadata = loadMetadata(dep, dir);
  var binaryLookup = metadata.bin || {};
  var binaries = toKeyValuePair(binaryLookup);

  return { dep: dep, binaries: binaries };
}

function getBinFeatures(dep, bin) {
  var binPath = _path2.default.join('node_modules', dep, bin.value).replace(/\\/g, '/');

  var features = [bin.key, '$(npm bin)/' + bin.key, 'node_modules/.bin/' + bin.key, './node_modules/.bin/' + bin.key, binPath, './' + binPath];

  return features;
}

function isUsedBin(dep, bin, scripts) {
  var features = getBinFeatures(dep, bin);
  return scripts.some(function (script) {
    return features.some(function (char) {
      return (' ' + script + ' ').indexOf(' ' + char + ' ') !== -1;
    });
  });
}

function getUsedDeps(deps, scripts, dir) {
  return deps.map(function (dep) {
    return toMetadata(dep, dir);
  }).filter(function (metadata) {
    return metadata.binaries.some(function (bin) {
      return isUsedBin(metadata.dep, bin, scripts);
    });
  }).map(function (metadata) {
    return metadata.dep;
  });
}

exports.default = function (content, filepath, deps, dir) {
  var scripts = (0, _getScripts2.default)(filepath, content);
  return getUsedDeps(deps, scripts, dir);
};

module.exports = exports['default'];